package com.ruoyi.project.workflow.common;

import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.project.workflow.domain.base.TableInfo;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.util.IOUtils;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.lang.reflect.Field;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ExcelBase<T> {

    private static final Logger log = LoggerFactory.getLogger(ExcelUtil.class);

    /**
     * 工作薄对象
     */
    private Workbook wb;
    /**
     * 工作表对象
     */
    private Sheet sheet;

    /**
     * 样式列表
     */
    private Map<String, CellStyle> styles;

    /**
     * 当前行号
     */
    private int rownum;

    /**
     * 显示信息
     */
    private List<TableInfo> infoList;

    /**
     * 显示数据
     */
    private List<T> dataList;

    /*
     *数据有无
     */
    private Boolean dataFlg;

    /**
     * 实体对象
     */
    private Class<T> clazz;

    public ExcelBase(Class<T> clazz)
    {
        this.clazz = clazz;
    }

    public void init(List<TableInfo> iList, List<T> dList, String sheetName) {
        wb = new SXSSFWorkbook(500);
        sheet = wb.createSheet();
        wb.setSheetName(0, sheetName);
        styles = createStyles(wb);
        rownum = 0;
        infoList = iList;
        dataList = dList;
    }

    /*
    * 数据有无
    * */
    private void checkData(){
        if (infoList == null || infoList.size() == 0) {
            dataFlg = false;
        } else if (dataList == null || dataList.size() == 0) {
            dataFlg = false;
        } else {
            dataFlg = true;
        }
    }

    /**
     * 创建excel第一行标题
     */
    private void creatData(){
        Row subRow = sheet.createRow(rownum);
        for (TableInfo info : infoList) {
            Cell headCell = subRow.createCell(info.getIndex().intValue() - 1);
            headCell.setCellValue(info.getValue());
            headCell.setCellStyle(styles.get("title"));
        }
        rownum++;
    }

    /**
     * 创建excel数据
     */
    private void creatValue() {
        for (T item: dataList) {
            Row subRow = sheet.createRow(rownum);
            for (TableInfo info : infoList) {
                Cell headCell = subRow.createCell(info.getIndex().intValue() - 1);
                try {
                    Field field = clazz.getDeclaredField(info.getLabel());
                    field.setAccessible(true);
                    Object getValue = field.get(item);
                    if (getValue != null) {
                        headCell.setCellValue(getValue.toString());
                    }
                } catch (Exception e) {
                    log.error("导出Excel异常{}", e.getMessage());
                }
            }
            rownum++;
        }
    }

    /**
     * 创建表格样式
     *
     * @param wb 工作薄对象
     * @return 样式列表
     */
    private Map<String, CellStyle> createStyles(Workbook wb)
    {
        // 写入各条记录,每条记录对应excel表中的一行
        Map<String, CellStyle> styles = new HashMap<String, CellStyle>();
        CellStyle style = wb.createCellStyle();
        style.setAlignment(HorizontalAlignment.CENTER);
        style.setVerticalAlignment(VerticalAlignment.CENTER);
        style.setFillForegroundColor(HSSFColor.HSSFColorPredefined.YELLOW.getIndex());
        style.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        Font titleFont = wb.createFont();
        titleFont.setFontName("Calibri");
        titleFont.setFontHeightInPoints((short) 10);
        titleFont.setBold(true);
        style.setFont(titleFont);
        styles.put("title", style);

        return styles;
    }

    public void exportExcel(HttpServletResponse response, List<TableInfo> iList,
                            List<T> dlist, String sheetName, String excelName) {
        init(iList, dlist, sheetName);
        checkData();
        if (dataFlg) {
            creatData();
            creatValue();
            writeSheet(response, excelName);
        } else {
            IOUtils.closeQuietly(wb);
        }
    }

    private void writeSheet(HttpServletResponse response, String excelName) {
        InputStream in = null;
        OutputStream out = null;
        response.setContentType("application/octet-stream");
        response.setCharacterEncoding("utf-8");
        try {
            response.addHeader("Access-Control-Expose-Headers", "Content-Disposition");
            response.addHeader("Content-Disposition","attachment;filename=" +
                    URLEncoder.encode(excelName,"utf-8"));
            ByteArrayOutputStream os = new ByteArrayOutputStream();
            wb.write(os);
            in = new ByteArrayInputStream(os.toByteArray());
            response.addHeader("Content-Length", String.valueOf(in.available()));

            out = response.getOutputStream();
            byte[] b = new byte[in.available()];
            int len;
            while ((len = in.read(b)) > 0) {
                out.write(b,0, len);
            }
            out.flush();
            out.close();
            in.close();
        } catch (Exception e) {
            log.error("导出Excel异常{}", e.getMessage());
        } finally {
            IOUtils.closeQuietly(wb);
            try {
                if(in != null) {
                    in.close();
                }
                if (out != null) {
                    out.close();
                }
            } catch (IOException e) {
                log.error("导出Excel异常{}", e.getMessage());
            }
        }
    }
}
